package main

import (
	"bufio"
	"bytes"
	"encoding/json"
	"flag"
	"fmt"
	"log"
	"net/http"
	"sort"
	"strconv"
	"strings"
)

var listen = flag.String("listen", "127.0.0.1:8080", "监听地址")

func main() {
	flag.Parse()
	mux := http.NewServeMux()
	mux.HandleFunc("/", APITransfer)

	log.Printf("Service listen and serve at %s\n", *listen)
	http.ListenAndServe(*listen, mux)
}

func JSONResp(w http.ResponseWriter, v interface{}, statusCode int) {
	// http 返回 json 的包装函数
	// 从 golang 的结构体转换称字符串
	buf := &bytes.Buffer{}
	enc := json.NewEncoder(buf)
	enc.SetEscapeHTML(true)
	if err := enc.Encode(v); err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	w.Header().Set("Content-Type", "application/json; charset=utf-8")
	w.WriteHeader(statusCode)
	_, _ = w.Write(buf.Bytes())
}

func APITransfer(w http.ResponseWriter, r *http.Request) {
	// 从目标地址获取内容
	oriresp, err := http.Get("https://pcdn-oss.corp.kuaishou.com/provider/bendy93687365/public/resgap.txt")
	if err != nil {
		JSONResp(w, err.Error(), http.StatusBadGateway)
		return
	}

	// oriresp.Body 是响应的 http 内容
	// 做成 scanner 扫描器，一行行的读取
	defer oriresp.Body.Close()
	sc := bufio.NewScanner(oriresp.Body)
	sc.Split(bufio.ScanLines)

	// 初始化接口响应内容
	data := []struct {
		NetLTD string `json:"netltd"`
		Needs  []struct {
			Province string  `json:"province"`
			Number   float64 `json:"number"`
		} `json:"needs"`
	}{{NetLTD: "电信"}, {NetLTD: "联通"}, {NetLTD: "移动"}}

	var annotation strings.Builder
	for sc.Scan() {
		// 扫描一行
		line := strings.TrimSpace(sc.Text())
		// 如果以 "#" 开头，忽略
		if strings.HasPrefix(line, "#") {
			annotation.WriteString(line + "\n")
			continue
		}

		parts := strings.Fields(line)
		// 如果一行不是五个元素，返回错误
		if len(parts) != 5 {
			JSONResp(w, fmt.Sprintf("原始结构错误: %v 需要 %d/5 个元素 ", parts, len(parts)), http.StatusBadGateway)
			return
		}

		// 如果第一个元素是 "id" ，为表头，确认表头正确
		if parts[0] == "id" {
			if parts[1] == "省份" && parts[2] == "电信缺口" && parts[3] == "联通缺口" && parts[4] == "移动缺口" {
				continue
			}
			JSONResp(w, fmt.Sprintf("原始结构错误：[%s] 表头错误", line), http.StatusBadGateway)
			return
		}

		for index, num := range parts[2:] {
			// 从第三个元素开始，是真实数据
			if num == "N" {
				// 如果数据为 N ，记 0
				data[index].Needs = append(data[index].Needs, struct {
					Province string  `json:"province"`
					Number   float64 `json:"number"`
				}{parts[1], 0})
				continue
			}

			// 将数据从字符串转为 int
			num, err := strconv.ParseFloat(num, 10)
			if err != nil {
				JSONResp(w, parts[1]+" 原始结构错误: "+err.Error(), http.StatusBadGateway)
				return
			}

			data[index].Needs = append(data[index].Needs, struct {
				Province string  `json:"province"`
				Number   float64 `json:"number"`
			}{parts[1], num})
		}
	}

	// 比较函数，这是一个返回「闭包函数」的函数，
	// 从提供的 index ，生成对应 网络运营商 返回结构 的比较函数
	sortcomp := func(index int) func(i, j int) bool {
		return func(i, j int) bool {
			return data[index].Needs[i].Number > data[index].Needs[j].Number
		}
	}

	// 使用比较函数对三个运营商的数据进行排序
	sort.Slice(data[0].Needs, sortcomp(0))
	sort.Slice(data[1].Needs, sortcomp(1))
	sort.Slice(data[2].Needs, sortcomp(2))

	JSONResp(w, map[string]any{
		"annotation": annotation.String(),
		"data":       data,
	}, http.StatusOK)
}
